package com.incubator.common.net.netty.server;

import com.incubator.common.log4j.Log;
import com.incubator.common.thread.NamedThreadFactory;
import io.netty.channel.EventLoopGroup;
import io.netty.channel.ServerChannel;
import io.netty.channel.epoll.EpollEventLoopGroup;
import io.netty.channel.epoll.EpollServerSocketChannel;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.nio.NioServerSocketChannel;

import java.util.concurrent.TimeUnit;

/**
 * NettyServerConfig
 */
public class NettyServerConfig {

	private static final int DEFAULT_BOSS_THREADS = Math.max(1, Runtime.getRuntime().availableProcessors() / 2);
	private static final int DEFAULT_IO_THREADS = Math.max(1, Runtime.getRuntime().availableProcessors() * 2);

	protected final Class<? extends ServerChannel> channelClass;

	protected final EventLoopGroup boss;

	protected final EventLoopGroup ioWorkers;

	public NettyServerConfig(Class<? extends ServerChannel> channelClass, EventLoopGroup boss, EventLoopGroup ioWorkers) {
		this.channelClass = channelClass;
		this.boss = boss;
		this.ioWorkers = ioWorkers;
	}

	public NettyServerConfig() {
		this(getDefaultChannelClass(),
				getEventLoopGroup(DEFAULT_BOSS_THREADS, "Server-boss"),
				getEventLoopGroup(DEFAULT_IO_THREADS, "Server-ioWorkers"));
	}

	public NettyServerConfig(int bossThreads, int ioThreads) {
		this(getDefaultChannelClass(),
				getEventLoopGroup(bossThreads, "Server-boss"),
				getEventLoopGroup(ioThreads, "Server-ioWorkers"));
	}

	public Class<? extends ServerChannel> getChannelClass() {
		return channelClass;
	}

	public EventLoopGroup getBoss() {
		return boss;
	}

	public EventLoopGroup getIoWorkers() {
		return ioWorkers;
	}

	/**
	 * 获取默认的 Channel 类型，根据操作系统选择。
	 */
	public static Class<? extends ServerChannel> getDefaultChannelClass() {
		if (isLinuxPlatform()) {
			return EpollServerSocketChannel.class;
		}
		return NioServerSocketChannel.class;
	}

	/**
	 * 获取合适的 EventLoopGroup 实例，根据操作系统选择 Nio 或 Epoll。
	 */
	private static EventLoopGroup getEventLoopGroup(int threads, String namePrefix) {
		if (isLinuxPlatform() && isEpollAvailable()) {
			// Linux 系统且支持 Epoll 时，使用 EpollEventLoopGroup
			return new EpollEventLoopGroup(threads, new NamedThreadFactory(namePrefix, true));
		} else {
			// 其他平台使用 NioEventLoopGroup
			return new NioEventLoopGroup(threads, new NamedThreadFactory(namePrefix, true));
		}
	}

	/**
	 * 判断是否为 Linux 平台。
	 */
	private static boolean isLinuxPlatform() {
		String osName = System.getProperty("os.name").toLowerCase();
		return osName.contains("linux");
	}

	/**
	 * 判断是否支持 Epoll。
	 */
	private static boolean isEpollAvailable() {
		try {
			// 尝试加载 Epoll 相关类来检测是否支持 Epoll
			Class.forName("io.netty.channel.epoll.Epoll");
			return true;
		} catch (ClassNotFoundException e) {
			return false;
		}
	}

	/**
	 * 释放资源。
	 */
	public void destroy() {
		try {
			if (boss != null) {
				boss.shutdownGracefully(2, 30, TimeUnit.SECONDS).sync();
			}
		} catch (InterruptedException e) {
			Thread.currentThread().interrupt();
			Log.error("Error shutting down boss group: " + e.getMessage());
		}

		try {
			if (ioWorkers != null) {
				ioWorkers.shutdownGracefully(2, 30, TimeUnit.SECONDS).sync();
			}
		} catch (InterruptedException e) {
			Thread.currentThread().interrupt();
			Log.error("Error shutting down IO workers group: " + e.getMessage());
		}
	}
}
