package com.incubator.game.robot;

import com.incubator.game.GGame;
import com.incubator.game.data.po.PlayerInfoPO;
import com.incubator.game.player.Player;
import java.util.Queue;
import java.util.concurrent.ConcurrentLinkedQueue;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

/**
 * 机器人对象池
 */
public final class RobotPool {

    private final Queue<Player> robotPool;
    private final ScheduledExecutorService cleaner;

    public RobotPool() {
        this.robotPool = new ConcurrentLinkedQueue<>();
        this.cleaner = Executors.newSingleThreadScheduledExecutor();
        // 定期清理空闲机器人
        this.cleaner.scheduleAtFixedRate(this::cleanIdleRobots, 10, 30, TimeUnit.SECONDS);
    }

    /**
     * 获取机器人对象
     */
    public Player acquireRobot(PlayerInfoPO playerInfoPO) {
        Player robot = this.robotPool.poll();
        if (robot == null) {
            robot = new Player(playerInfoPO);
        }
        robot.data = playerInfoPO;
        return robot;
    }

    /**
     * 归还机器人对象
     */
    public void releaseRobot(Player robot) {
        if (robot != null) {
            // 重置数据
            robot.data = new PlayerInfoPO();
            // 放回池中
            this.robotPool.offer(robot);
        }
    }

    /**
     * 定期清理空闲机器人
     */
    private void cleanIdleRobots() {
        while (this.robotPool.size() > GGame.maxIdlePlayers) {
            Player robot = this.robotPool.poll();
            if (robot != null) {
                // 释放资源
                this.releaseRobot(robot);
            }
        }
    }

    /**
     * 关闭对象池
     */
    public void shutdown() {
        this.cleaner.shutdownNow();
        for (Player robot : this.robotPool) {
            if (robot != null) {
                this.releaseRobot(robot);
            }
        }
        this.robotPool.clear();
    }

    public int getAvailableRobots() {
        return this.robotPool.size();
    }

}
